#! /usr/bin/python
from twisted.spread import pb
from ctlib.lib.wrapper import OptionPutEuropeanSingle, \
    OptionCallEuropeanSingle, OptionPutAmericanSingle, \
    OptionCallAmericanSingle

class SOptionPutEuropeanSingle(pb.Referenceable,OptionPutEuropeanSingle):
    def __init__(self,S,E,tau,r,q,sig):
        OptionPutEuropeanSingle.__init__(self,S,E,tau,r,q,sig)

    def remote_value(self):
        return self.value()


class SOptionCallEuropeanSingle(pb.Referenceable,OptionCallEuropeanSingle):
    def __init__(self,S,E,tau,r,q,sig):
        OptionCallEuropeanSingle.__init__(self,S,E,tau,r,q,sig)

    def remote_value(self):
        return self.value()


class SOptionPutAmericanSingle(pb.Referenceable,OptionPutAmericanSingle):
    def __init__(self,S,E,tau,r,q,sig):
        OptionPutAmericanSingle.__init__(self,S,E,tau,r,q,sig)

    def remote_value(self):
        return self.value()


class SOptionCallAmericanSingle(pb.Referenceable,OptionCallAmericanSingle):
    def __init__(self,S,E,tau,r,q,sig):
        OptionCallAmericanSingle.__init__(self,S,E,tau,r,q,sig)

    def remote_value(self):
        return self.value()


#This is the important class
class SPortfolio(pb.Referenceable):
    def __init__(self,derivatives):
        self.derivatives = derivatives
        #put parallelism here

    def remote_value(self):
        value = 0.0
        for der in self.derivatives:
            if der[0] == "OptionCallEuropeanSingle":
                op = SOptionCallEuropeanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                value += op.value()

            if der[0] == "OptionPutEuropeanSingle":
                op = SOptionPutEuropeanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                value += op.value()

            if der[0] == "OptionCallAmericanSingle":
                op = SOptionCallAmericanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                value += op.value()

            if der[0] == "OptionPutAmericanSingle":
                op = SOptionCallAmericanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                value += op.value()            

            print "Computed a " + der[0] + " value"

        return value

    def remote_delta(self):
        delta = list()
        for der in self.derivatives:
            if der[0] == "OptionCallEuropeanSingle":
                op = SOptionCallEuropeanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                delta.append(op.delta())

            if der[0] == "OptionPutEuropeanSingle":
                op = SOptionPutEuropeanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                delta.append(op.delta())

            if der[0] == "OptionCallAmericanSingle":
                op = SOptionCallAmericanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                delta.append(op.delta())

            if der[0] == "OptionCallAmericanSingle":
                op = SOptionCallAmericanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                delta.append(op.delta())

            print "Computed a " + der[0] + " delta"

        return delta

    def remote_gamma(self):
        gamma = 0.0

        for der in self.derivatives:
            if der(0) == "OptionCallEuropeanSingle":
                op = SOptionCallEuropeanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                gamma += op.gamma()

            if der(0) == "OptionPutEuropeanSingle":
                op = SOptionPutEuropeanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                gamma += op.gamma()

            if der(0) == "OptionCallAmericanSingle":
                op = SOptionCallAmericanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                gamma += op.gamma()

            if der(0) == "OptionCallAmericanSingle":
                op = SOptionCallAmericanSingle(der[1],der[2],der[3],der[4],der[5],der[6])
                gamma += op.gamma()

            print "Computed a " + der[0] + " gamma"

        return gamma
        

class Dispatcher(pb.Root):
    def __init__(self):
        self.two = None

    def remote_OptionPutEuropeanSingle(self,S,E,tau,r,q,sig):
        op = SOptionPutEuropeanSingle(S,E,tau,r,q,sig)
        print "returning a single asset European Put Option", op
        return op

    def remote_OptionCallEuropeanSingle(self,S,E,tau,r,q,sig):
        op = SOptionCallEuropeanSingle(S,E,tau,r,q,sig)
        print "returning a single asset European Call Option", op
        return op

    def remote_OptionPutAmericanSingle(self,S,E,tau,r,q,sig):
        op = SOptionPutAmericanSingle(S,E,tau,r,q,sig)
        print "returning a single asset American Put Option", op
        return op

    def remote_OptionCallAmericanSingle(self,S,E,tau,r,q,sig):
        op = SOptionCallAmericanSingle(S,E,tau,r,q,sig)
        print "returning a single asset American Call Option", op
        return op

    def remote_Portfolio(self,portfolio):
        op = SPortfolio(portfolio)
        print "returning a remote portfolio", op
        return op


if __name__ == '__main__':
    from twisted.internet import reactor
    reactor.listenTCP(8800, pb.PBServerFactory(Dispatcher()))
    reactor.run()
